@@ -462,10 +462,12 @@ gtk_gs_widget_event(GtkWidget *widget,
          only if event if client_event */
 
         gs->message_window = event->client.data.l[0];
-        g_print("Message window %ld\n", gs->message_window);
+        /* g_print("Message window %ld\n", gs->message_window); */
 
 	if (event->client.message_type == gs_class->page_atom) {
-                g_print("Interpreter completed: busy -> FALSE\n");
+                /*
+                  g_print("Interpreter completed: busy -> FALSE\n");
+                */
                 gs->busy = FALSE;
 	}
         return TRUE;
@@ -631,8 +633,9 @@ set_up_page (GtkGS *gs)
                 g_print("Do not create pixmap\n");
         }
         else if ((gs->use_bpixmap) && compute_size (gs)) { 
+                /*
                 g_print("Creating pixmap\n");
-
+                */
                 gdk_flush(); 
 
                 if (gs->bpixmap) {
@@ -663,7 +666,7 @@ set_up_page (GtkGS *gs)
                  gs->bottom_margin,
                  gs->right_margin,
                  gs->top_margin);
-        g_print("Sent to PostScript %s\n",buf);
+        /*        g_print("Sent to PostScript %s\n",buf); */
         gdk_property_change(gs->pstarget,
                             gs_class->gs_atom,
                             gs_class->string_atom,
@@ -686,8 +689,8 @@ close_pipe(int p[2])
 static gboolean
 is_interpreter_ready(GtkGS *gs)
 {
-        g_print("Is interpreter ready pid %d busy %d %lx [%s]", 
-                 gs->interpreter_pid,gs->busy, gs->ps_input,gs->ps_input);
+        /*        g_print("Is interpreter ready pid %d busy %d %lx [%s]",  
+                  gs->interpreter_pid,gs->busy, gs->ps_input,gs->ps_input); */
         return (gs->interpreter_pid != -1 &&
                 !gs->busy &&
                 gs->ps_input == NULL);
@@ -818,6 +821,7 @@ input (gpointer data, gint source, GdkInputCondition condition)
                                 gs->input_buffer_ptr += bytes_written;
                         }
                 }
+                while (g_main_iteration (FALSE)) {}
         } while(gs->ps_input &&
                 gs->buffer_bytes_left == 0);
         
@@ -1111,7 +1115,9 @@ gtk_gs_load (GtkGS *gs, char *fname)
         struct stat sbuf;
 	
 	g_assert(gs != NULL);
+        /*
         printf (" in gs_load\n");
+        */
 	if (fname == NULL) {
                 if(gdk_window_is_visible(gs->pstarget))
                         gdk_window_hide(gs->pstarget);
@@ -1140,7 +1146,9 @@ gtk_gs_load (GtkGS *gs, char *fname)
 
                 gs->gs_filename = g_strdup(fname);
 
-                printf (" filename is: %s\n", gs->gs_filename);
+                /*
+                  printf (" filename is: %s\n", gs->gs_filename);
+                */
 		gs->gs_filename_raw = g_strdup(fname);
                 gs->current_page = -1;
 
@@ -1158,6 +1166,8 @@ gtk_gs_load (GtkGS *gs, char *fname)
                         
                         /* We have to report error to the reader, do we do it here */
                         g_print("File does not seem to be PostScript\n");
+                        /*
+
                         if (gs->gs_filename)
                                 g_free(gs->gs_filename);
                         if (gs->gs_filename_raw)
@@ -1175,12 +1185,16 @@ gtk_gs_load (GtkGS *gs, char *fname)
                         gs->ury = 0;
                         if(gdk_window_is_visible(gs->pstarget))
                                 gdk_window_hide(gs->pstarget);
-
                         return FALSE;
+                        */
+                        gs->structured_doc = FALSE; 
+                        gs->send_filename_to_gs = TRUE;
+
                 }
 
+                /*
                 g_print("Page order %d\n", gs->doc->pageorder);
-
+                */
                 if ((!gs->doc->epsf && gs->doc->numpages>0) || 
                     (gs->doc->epsf && gs->doc->numpages>1)) {
                         gs->structured_doc = TRUE; 
@@ -1193,8 +1207,10 @@ gtk_gs_load (GtkGS *gs, char *fname)
                         gs->send_filename_to_gs = TRUE;
                 }
 
+                /*
                 g_print(gs->structured_doc?"Struc, pages[%d]\n":"Non struc pages[%d]\n",
                         gs->doc->numpages);
+                */
 
                 stat(gs->gs_filename, &sbuf);
                 gs->mtime = sbuf.st_mtime;
@@ -1226,15 +1242,21 @@ gtk_gs_next_page(GtkGS *gs)
 	gs->busy = TRUE;
 	pw = (GdkWindowPrivate *)gs->widget.window;
 
+        /*
         g_print("Sending NEXT message to %ld\n", gs->message_window);
+        */
 	event.xclient.type          = ClientMessage;
 	event.xclient.display       = gdk_display;
 	event.xclient.window        = gs->message_window;
 	event.xclient.message_type  = gs_class->next_atom;
 	event.xclient.format        = 32;
+        /*
         g_print ("sending event.");
+        */
 	gdk_send_xevent(gs->message_window, FALSE, 0, &event);
+        /*
         g_print ("..sent\n");
+        */
         gdk_flush();
 	return TRUE;
 }
@@ -1245,7 +1267,9 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
         struct stat sbuf;
 	g_assert (gs != NULL);
 
+        /*
         g_print("gs goto page %d\n", page);
+        */
         if (!gs->gs_filename) {
                 g_print ("filename is null???\n");
                 return FALSE;
@@ -1290,10 +1314,10 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 
 
                 if (is_interpreter_ready (gs)) {
-                       g_print ("the interpreter is ready!\n");
+                        /*g_print ("the interpreter is ready!\n"); */
                         gtk_gs_next_page (gs);
                 } else {
-                        g_print ("the interpreter is not ready -- starting now\n");
+                        /* g_print ("the interpreter is not ready -- starting now\n"); */
                         gtk_gs_enable_interpreter (gs);
                         send_ps(gs, gs->doc->beginprolog, 
                                 gs->doc->lenprolog, FALSE);
@@ -1317,14 +1341,16 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 g_print("sending page [non structured]\n");
 
                 if (is_interpreter_ready (gs)) {
-                       g_print ("the interpreter is ready!\n");
+                        /* g_print ("the interpreter is ready!\n"); */
                 } else {
-                        g_print ("the interpreter is not ready -- starting now\n");
+                        /* g_print ("the interpreter is not ready -- starting now\n"); */
                         gtk_gs_enable_interpreter (gs);
+                        /*
                         send_ps(gs, gs->doc->beginprolog, 
                                 gs->doc->lenprolog, FALSE);
                         send_ps(gs, gs->doc->beginsetup, 
                                 gs->doc->lensetup, FALSE);
+                        */
                 }
                 gtk_gs_next_page (gs);
         }
@@ -1371,7 +1397,7 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
                 }
         }       
 
-        g_print("page media %d\n", new_pagemedia);
+        /*  g_print("page media %d\n", new_pagemedia); */
         
         /* Compute bounding box */
         if (gs->doc && gs->doc->epsf 
@@ -1402,11 +1428,11 @@ gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid)
                 gs->urx = new_urx;
                 gs->ury = new_ury;
                 changed = TRUE;
-                g_print ("setting up the page l: %d l:%d u:%d u:%d\n",new_llx,new_lly,new_urx,new_ury);
+                /* g_print ("setting up the page l: %d l:%d u:%d u:%d\n",new_llx,new_lly,new_urx,new_ury); */
 
                 set_up_page(gs);
         }
-        g_print ("\tall done.\n");
+        /* g_print ("\tall done.\n"); */
 	if(changed) {
 		gtk_widget_queue_resize (&(gs->widget));
 	}
@@ -1435,7 +1461,7 @@ void
 gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
 	g_assert (gs != NULL);
 
-        g_print ("zoom factor is %f\n",zoom);
+        /* g_print ("zoom factor is %f\n",zoom); */
         
         if (gs->zoom_factor != zoom) { 
                 gs->zoom_factor = zoom;
@@ -1490,7 +1516,7 @@ compute_size(GtkGS *gs)
         change = (new_width != gs->width * gs->zoom_factor) || (new_height != gs->height * gs->zoom_factor);
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
-        g_print("GS: width %d height %d \n", gs->width, gs->height);
+        /*  g_print("GS: width %d height %d \n", gs->width, gs->height); */
         if(GTK_WIDGET_REALIZED(gs)) {
                 if(!gdk_window_is_visible(gs->pstarget) && gs->width > 0 && gs->height > 0)
                         gdk_window_show(gs->pstarget);
