@@ -8,7 +8,6 @@
 #ifndef __GTK_GS_H__
 #define __GTK_GS_H__
 
-
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
 #include <errno.h>
@@ -87,7 +86,8 @@ struct _GtkGS
         gchar      *input_buffer;
         gint       gs_scanstyle;
 	time_t     mtime;
-
+        gboolean    send_filename_to_gs;     /* True if gs should read from file directly */
+        gboolean    reading_from_pipe;        /* True if ggv is reading input from pipe */
         struct document *doc;
 
         /* User selected options... */
@@ -96,8 +96,8 @@ struct _GtkGS
         gint default_page_media;
         gboolean override_media;
         gchar      *gs_interpreter;           /* /usr/bin/gs */
-	gchar      *gs_gs_cmd_scan_pdf;       /* gs -sPDFname..... */
-	gchar      *gs_uncompress_command;    /* set to gzip -cd */
+	gchar      *gs_scan_pdf_cmd;          /* gs -sPDFname..... */
+	gchar      *gs_uncompress_cmd;        /* set to gzip -cd */
         gfloat     xdpi, ydpi;
         gint       orientation;
 };
@@ -134,10 +134,11 @@ gboolean gtk_gs_load(GtkGS *gv, gchar *fname);
 
 /* control functions */
 void gtk_gs_center_page (GtkGS *gs);
+void gtk_gs_scroll (GtkGS *gs, gint, gint);
 gboolean gtk_gs_next_page(GtkGS *gs);
 gboolean gtk_gs_prev_page(GtkGS *gs);
 gboolean gtk_gs_goto_page(GtkGS *gs, gint);
-gboolean gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gboolean set_default, gboolean set_override, gint pageid);
+gboolean gtk_gs_set_pagemedia(GtkGS *gs, gint new_pagemedia, gint pageid);
 gboolean gtk_gs_set_orientation(GtkGS *gs, gint orientation);
 void gtk_gs_set_zoom(GtkGS *gs, gfloat zoom);
 gint gtk_gs_enable_interpreter(GtkGS *gs);
