@@ -8,6 +8,9 @@
 #include <gdk/gdk.h>
 #include <gnome.h>
 #include <math.h>
+#include <sys/stat.h>
+#include <libgnome/gnome-history.h>
+
 #include <gdk/gdkkeysyms.h>
 #include "gtkscrollpane.h"
 #include "gtkgs.h"
@@ -59,6 +62,8 @@ struct _ggv_window {
         GtkWidget *zoomoutbutton;
         GtkWidget *panel_vis, *menus_vis;           /* menu/side panel visibility check items in menus */
         GtkWidget *panel_vis_pu, *menus_vis_pu;     /* and in popup menu */
+        gboolean bRecentHasBeenInstantiated;        /* Recent menu has been intantiated? so we can
+                                                       remove it before we add new entries */
 };
 
 struct _ggv_prefs
@@ -125,6 +130,174 @@ static double compute_zoom(gint zoom_spec);
 static void prefs_destroy_callback(GtkWidget *w, ggv_prefs *p);
 static void prefs_apply_callback(GtkWidget *w, gint page, ggv_prefs *p);
 static void prefs_changed_callback(GtkWidget *w, ggv_prefs *p);
+static void zoomin_callback(GtkWidget *widget, gpointer *data);
+static void zoomout_callback(GtkWidget *widget, gpointer *data);
+static void recent_add(char *filename);
+static void recent_update_menus (ggv_window *ggv, GList *recent_files);
+static void recent_cb(GtkWidget *w, gpointer *data);
+
+static void
+recent_cb(GtkWidget *w, gpointer *data)
+{
+        char *ErrorMessage;
+        window_with_data *wwd = (window_with_data *) data;
+        struct stat StatRecord;
+
+        /*
+        g_print("TO open file %s\n",(char*)wwd->data);
+        */
+        if (stat((char*)wwd->data,&StatRecord) == 0) {
+                if (S_ISREG(StatRecord.st_mode)) {
+                        wwd->ggv->loaded = load_gs(wwd->ggv, (char*)wwd->data);
+                        return;
+                }
+                ErrorMessage = g_strdup_printf(_("%s is not a file."), (char*)wwd->data);
+
+        }
+        else {
+                ErrorMessage = g_strdup_printf(_("%s does not exist."), (char*)wwd->data);
+        }
+
+        error_message(wwd->ggv,ErrorMessage);
+
+        /*        g_free((char*)wwd->data); */
+}
+
+
+static void
+recent_update_menus (ggv_window *ggv, GList *recent_files)
+{
+	GnomeUIInfo *menu;
+        /*	gE_data *data; */
+	gchar *path;
+	int i;
+        window_with_data *wwd;
+
+        /*
+        g_print("Num recent %d\n", gs_num_recent);
+        */
+	if (gs_num_recent && ggv->bRecentHasBeenInstantiated) {
+                gnome_app_remove_menu_range (GNOME_APP(ggv->main_window), 
+                                             _("_File/"), 4, gs_num_recent + 1);
+                g_print("Removing...\n");
+        }
+
+	if (recent_files == NULL)
+		return;
+
+
+	/* insert a separator at the beginning */
+	
+	menu = g_malloc0 (2 * sizeof (GnomeUIInfo));
+	path = g_new (gchar, strlen (_("_File")) + strlen ("<Separator>") + 3 );
+	sprintf (path, "%s/%s", _("_File"), "<Separator>");
+	menu->type = GNOME_APP_UI_SEPARATOR;
+
+	(menu + 1)->type = GNOME_APP_UI_ENDOFINFO;
+	gnome_app_insert_menus (GNOME_APP(ggv->main_window), path, menu);
+
+
+	for (i = g_list_length (recent_files) - 1; i >= 0;  i--)
+	{
+                wwd = g_malloc0(sizeof(window_with_data));
+
+		menu = g_malloc0 (2 * sizeof (GnomeUIInfo));
+	
+		/* data = g_malloc0 (sizeof (gE_data));
+                   data->temp1 = g_strdup (g_list_nth_data (recent_files, i)); */
+		/*data->window = window;*/
+		menu->label = g_new (gchar, 
+			strlen (g_list_nth_data (recent_files, i)) + 5);
+		sprintf (menu->label, "_%i. %s", i+1, (gchar*)g_list_nth_data (recent_files, i));
+		menu->type = GNOME_APP_UI_ITEM;
+		menu->hint = NULL;
+                
+		menu->moreinfo = (gpointer) recent_cb;
+
+                wwd->data = (int)g_strdup(g_list_nth_data (recent_files, i));
+                wwd->ggv = ggv;
+
+		menu->user_data = wwd;
+		menu->unused_data = NULL;
+		menu->pixmap_type = 0;
+		menu->pixmap_info = NULL;
+		menu->accelerator_key = 0;
+
+		(menu + 1)->type = GNOME_APP_UI_ENDOFINFO;
+	
+		gnome_app_insert_menus (GNOME_APP(ggv->main_window), path, menu);
+		g_free (g_list_nth_data (recent_files, i));
+	}
+	gs_num_recent = g_list_length (recent_files);
+
+        ggv->bRecentHasBeenInstantiated = TRUE;
+
+
+
+	g_list_free (recent_files);
+}
+
+
+void recent_update (ggv_window *ggv)
+{
+	GList *filelist = NULL;
+	GList *dirlist = NULL;
+	
+	GList *gnome_recent_list;
+	GnomeHistoryEntry histentry;
+	char *filename;
+	int i, j;
+	gboolean duplicate = FALSE;
+	
+	filelist = NULL;
+	gnome_recent_list = gnome_history_get_recently_used ();
+	
+
+	if (g_list_length (gnome_recent_list) > 0)                {
+                for (i = g_list_length (gnome_recent_list) - 1; i >= 0; i--) {
+                        histentry = g_list_nth_data (gnome_recent_list, i);
+			if (strcmp ("ggv", histentry->creator) == 0) {
+				/* This is to make sure you don't have more than one
+				   file of the same name in the recent list
+				*/
+				if (g_list_length (filelist) > 0)
+					for (j = g_list_length (filelist) - 1; j >= 0; j--)
+						if (strcmp (histentry->filename, g_list_nth_data (filelist, j)) == 0)
+							filelist = g_list_remove (filelist, g_list_nth_data (filelist, j));
+                                
+				filename = g_malloc0 (strlen (histentry->filename) + 1);
+				strcpy (filename, histentry->filename);
+				filelist = g_list_append (filelist, filename);
+                                
+                                /* For recent-directories, not yet fully implemented...
+                                   end_path = strrchr (histentry->filename, '/');
+                                   if (end_path)
+                                   {
+                                   for (i = 0; i < strlen (histentry->filename); i++)
+                                   if ( (histentry->filename + i) == end_path)
+                                   break;
+                                   directory = g_malloc0 (i + 2);
+                                   strcat (directory, histentry->filename, i);
+                                   }
+                                */
+				if (g_list_length (filelist) == MAX_RECENT)
+					break;
+			}
+		}
+	}
+	gnome_history_free_recently_used_list (gnome_recent_list);
+
+	gs_num_recent = g_list_length (filelist);
+
+	
+	recent_update_menus (ggv, filelist);
+}
+
+
+static void recent_add(char *filename)
+{
+        gnome_history_recently_used (filename, "text/plain", "ggv", "");
+}
 
 /* Useful functions */
 static gboolean
@@ -133,7 +306,12 @@ load_gs(ggv_window *ggv, char *fname)
 	char *title;
 	guint i,n;
 	gchar *texts[3], *flash;
+        gint page;
 
+        if(ggv->loaded && strcmp(fname, GTK_GS(ggv->gs)->gs_filename) == 0)
+                page = GTK_GS(ggv->gs)->current_page;
+        else
+                page = 0;
         flash = g_strdup_printf(_("Opening %s..."), fname);
         flash_message(ggv, flash);
 	if (!gtk_gs_load (GTK_GS (ggv->gs), fname)) {
@@ -194,10 +372,10 @@ load_gs(ggv_window *ggv, char *fname)
                         }
                 }
                 gtk_clist_thaw(GTK_CLIST(ggv->pagelist));
-		goto_page( ggv, GTK_GS(ggv->gs)->current_page );
+		goto_page( ggv, page );
         }
         else
-		goto_page( ggv, GTK_GS(ggv->gs)->current_page );
+		goto_page( ggv, page );
         gtk_widget_set_sensitive (ggv->zoomoutbutton, TRUE);
         gtk_widget_set_sensitive (ggv->zoominbutton, TRUE);
         gtk_widget_set_sensitive (ggv->printitem, TRUE);
@@ -370,17 +548,62 @@ static void file_open_ok_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
         gchar *file;
+        gchar *baseFileName;
+        char *ErrorMessage;
+        struct stat StatRecord;
 
 	file = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(ggv->file_sel)));
 
+        /* We have to verify that this file exists */
+        
+        if (stat(file,&StatRecord) == 0) {
+                
+                if (S_ISDIR(StatRecord.st_mode)) {
+                        /* This is a directory, reset window and
+                           try again */
+                        if (strlen(file) > 0 && file[strlen(file)-1] != '/')
+                                strcat(file,"/");
+                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(ggv->file_sel),
+                                                        file);
+                        return;
+                }
+                
+        }
+        else {
+                /* We have to check if there is a wildcard in the directory name
+                 */
+                
+                baseFileName = g_basename(file);
+                if (strchr(baseFileName, '?' ) != NULL ||
+                    strchr(baseFileName, '*' ) != NULL) {
+                        gtk_file_selection_complete(GTK_FILE_SELECTION(ggv->file_sel),
+                                                    file);
+                }
+                           
+                /* FIXME:  currently , if the file does not exist,
+                   the window does not close, but it does not send a message.
+                   If we send a message, the focus does not return to the
+                   File window after the user clicks on it
+
+                  ErrorMessage = g_strdup_printf(_("File does not exist %s."), file);
+                  error_message(ggv,ErrorMessage);
+                */
+                return;
+        }
         gtk_widget_destroy(ggv->file_sel);
         ggv->file_sel = NULL;
 
-        ggv->loaded = load_gs(ggv, file);
+        if ((ggv->loaded = load_gs(ggv, file))) {
+                recent_add (file);
+                recent_update (ggv);
 
+        }
         g_free(file);
 }
 
+
+
+
 static void
 show_menubar_callback(GtkWidget *widget, gpointer data)
 {
@@ -526,7 +749,6 @@ key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event, gpointer data)
                         }
                         break;
                 }
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_BackSpace:
         case GDK_Delete:
@@ -553,39 +775,43 @@ key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event, gpointer data)
                         }
                         break;
                 }
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Left:
                 gtk_scrollpane_step(GTK_SCROLLPANE (ggv->scrollpane),GTK_SCROLLPANE_SCROLL_LEFT,FALSE);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Right:
                 gtk_scrollpane_step(GTK_SCROLLPANE (ggv->scrollpane),GTK_SCROLLPANE_SCROLL_RIGHT,FALSE);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Up:
                 gtk_scrollpane_step(GTK_SCROLLPANE (ggv->scrollpane),GTK_SCROLLPANE_SCROLL_UP,FALSE);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Down:
                 gtk_scrollpane_step(GTK_SCROLLPANE (ggv->scrollpane),GTK_SCROLLPANE_SCROLL_DOWN,FALSE);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Page_Up:
 		goto_page( ggv, GTK_GS(ggv->gs)->current_page - 1);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
         case GDK_Page_Down:
 		goto_page( ggv, GTK_GS(ggv->gs)->current_page + 1);
-                gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
                 break;
+	case GDK_plus:
+		zoomin_callback(ggv->gs, (gpointer *)ggv);
+		break;
+	case GDK_minus:
+		zoomout_callback(ggv->gs, (gpointer *)ggv);
+		break;
+	default:
+		goto DONT_STOP_SIGNAL;
         }
+	gtk_signal_emit_stop_by_name (GTK_OBJECT(ggv->main_window), "key_press_event");
+ DONT_STOP_SIGNAL:
+	;
 }
 
 static void next_page_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
-
+        g_print("Going to next page [%d]\n", ggv->loaded);
         if (ggv->loaded) {
 		goto_page( ggv, GTK_GS(ggv->gs)->current_page + 1);
         }
@@ -595,6 +821,7 @@ static void next_page_callback(GtkWidget *widget, gpointer data)
 static void open_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
+        gchar *dirname;
 
         if(ggv->file_sel)
                 return;
@@ -609,6 +836,16 @@ static void open_callback(GtkWidget *widget, gpointer data)
 			   "clicked",
 			   GTK_SIGNAL_FUNC(file_open_ok_callback), data);
 
+        gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(ggv->file_sel));
+
+        /* Switch to the directory where the current 
+           file came from */
+
+        if (GTK_GS (ggv->gs)->gs_filename != NULL &&
+            (dirname=g_dirname(GTK_GS (ggv->gs)->gs_filename)) != NULL)
+                gtk_file_selection_complete(GTK_FILE_SELECTION(ggv->file_sel),
+                                                    dirname);
+
         gtk_widget_show(ggv->file_sel);
 }
 
@@ -643,7 +880,7 @@ static void print_callback(GtkWidget *widget, gpointer data)
                 print_command = g_strdup_printf(gs_print_cmd,
                                                 GTK_GS(ggv->gs)->gs_filename);
                 if(print_command) {
-                        system(print_command);
+                        gnome_execute_shell(NULL, print_command);
                         g_free(print_command);
                 }
         }
@@ -692,7 +929,8 @@ scrollpane_right_click_callback(GtkWidget *widget, gpointer data)
         }
 }
 
-static void select_page_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
+static void 
+select_page_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
 
@@ -703,7 +941,8 @@ static void select_page_callback(GtkWidget *widget, gint row, gint column, GdkEv
         }
 }
 
-static void drop_callback(GtkWidget *widget, GdkDragContext *context,
+static void 
+drop_callback(GtkWidget *widget, GdkDragContext *context,
                           gint x, gint y, GtkSelectionData *selection_data,
                           guint info, guint time, gpointer data) {
 	GList *names, *list;
@@ -892,50 +1131,47 @@ create_menus(ggv_window *retval, window_with_data *temp)
         GnomeUIInfo settingsMenu[] = {
                 GNOMEUIINFO_MENU_PREFERENCES_ITEM(preferences_callback, retval),
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_TOGGLEITEM, N_("Show Menus"), N_("Toggle menu visibility"), show_menubar_callback, retval, NULL,
+                {GNOME_APP_UI_TOGGLEITEM, N_("_Show Menus"), N_("Toggle menu visibility"), show_menubar_callback, retval, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_TOGGLEITEM, N_("Show Side Panel"), N_("Toggle side panel visibility"), hide_panel_callback, retval, NULL,
+                {GNOME_APP_UI_TOGGLEITEM, N_("S_how Side Panel"), N_("Toggle side panel visibility"), hide_panel_callback, retval, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 GNOMEUIINFO_END
         };
 
         GnomeUIInfo fileMenu[] = {
-                GNOMEUIINFO_MENU_NEW_ITEM(N_("New Window"), N_("Open a new window"), new_callback, NULL),
-                {GNOME_APP_UI_ITEM, N_("Open"), N_("Load a postscript document"), open_callback, retval, NULL,
+                GNOMEUIINFO_MENU_NEW_ITEM(N_("_New Window"), N_("Open a new window"), new_callback, NULL),
+                {GNOME_APP_UI_ITEM, N_("_Open"), N_("Load a postscript document"), open_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 'o', GDK_CONTROL_MASK, NULL},
-                {GNOME_APP_UI_ITEM, N_("Reload"), N_("Reload current document from disk"), reload_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Reload"), N_("Reload current document from disk"), reload_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REVERT, 'r', GDK_CONTROL_MASK, NULL},
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_ITEM, N_("Print..."), N_("Print the document"), print_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the document"), print_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT, 'p', GDK_CONTROL_MASK, NULL },
-                {GNOME_APP_UI_ITEM, N_("Save"), NULL, save_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Save"), NULL, save_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, N_("Save as..."), NULL, save_as_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("S_ave as..."), NULL, save_as_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_ITEM, N_("Recently Used Docs"), NULL, NULL, NULL, NULL,
-                 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN, 0, 0, NULL},
-                {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_ITEM, N_("Close"), N_("Close this window"), close_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Close"), N_("Close this window"), close_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 'w', GDK_CONTROL_MASK, NULL },
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_ITEM, N_("Exit"), N_("Exit GGv"), exit_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("E_xit"), N_("Exit GGv"), exit_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK, NULL },
                 {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL}
         };
         
         GnomeUIInfo documentMenu[] = {
-                {GNOME_APP_UI_ITEM, N_("Next Page"), N_("Show next page"), next_page_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Next Page"), N_("Show next page"), next_page_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, N_("Previous Page"), N_("Show previous page"), previous_page_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Previous Page"), N_("Show previous page"), previous_page_callback, retval, NULL,
                  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, 0, 0, NULL},
-                {GNOME_APP_UI_ITEM, N_("Recenter Page"), N_("Center page"), recenter_page_callback, retval, NULL,
+                {GNOME_APP_UI_ITEM, N_("_Recenter Page"), N_("Center page"), recenter_page_callback, retval, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
-                {GNOME_APP_UI_SUBTREE, N_("Orientation"), NULL, orientationMenu, NULL, NULL,
+                {GNOME_APP_UI_SUBTREE, N_("_Orientation"), NULL, orientationMenu, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
-                {GNOME_APP_UI_SUBTREE, N_("Zoom"), NULL, zoomMenu, NULL, NULL,
+                {GNOME_APP_UI_SUBTREE, N_("_Zoom"), NULL, zoomMenu, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_ENDOFINFO}
         };
@@ -998,7 +1234,7 @@ create_sidebar(ggv_window *retval)
         pic = gnome_stock_pixmap_widget_new(retval->prevbutton, GNOME_STOCK_MENU_BACK);
         gtk_widget_show (pic);
         gtk_container_add (GTK_CONTAINER (retval->prevbutton), pic);
-        pic = gnome_stock_pixmap_widget_new(retval->prevbutton, GNOME_STOCK_MENU_FORWARD);
+        pic = gnome_stock_pixmap_widget_new(retval->nextbutton, GNOME_STOCK_MENU_FORWARD);
         gtk_widget_show (pic);
         gtk_container_add (GTK_CONTAINER (retval->nextbutton), pic);
         gtk_signal_connect (GTK_OBJECT(retval->nextbutton), "clicked", GTK_SIGNAL_FUNC(next_page_callback), retval);
@@ -1188,7 +1424,7 @@ ggv_window *create_ggv_window()
                 gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(retval->panel_vis),
                                                 retval->show_panel);
         }
-
+        retval->bRecentHasBeenInstantiated = FALSE;
         return retval;
 }
 
@@ -1218,6 +1454,8 @@ open_window(gchar *filename)
                 active_ggv ++;
                 window_list = g_list_append(window_list, ggv);
                 gtk_widget_show (ggv->main_window);
+                recent_update(ggv);
+
         }
         else {
                 ggv_window *ggv = create_ggv_window ();
@@ -1458,7 +1696,7 @@ set_gs_prefs(ggv_window *ggv)
                 redisplay = TRUE;
         set_prefs(GTK_GS(ggv->gs));
         if(redisplay)
-                gtk_gs_goto_page(GTK_GS(ggv->gs), GTK_GS(ggv->gs)->current_page);
+                reload_callback(NULL, ggv);
 }
 
 static void
@@ -1508,3 +1746,4 @@ main(int argc, char *argv[])
 
         return 0;
 }
+
