@@ -119,6 +119,7 @@ static int  blank();
  *
  *      Header section: 
  *      Must start with %!PS-Adobe-.  Version numbers ignored.
+ *      Also allowed to be just %!PS, many files seem to have that.
  *
  *      %!PS-Adobe-* [EPSF-*]
  *      %%BoundingBox: <int> <int> <int> <int>|(atend)
@@ -323,7 +324,7 @@ psscan(FILE **fileP, const char *filename, const char *tmpprefix,
 
     /* Header comments */
 
-    if (iscomment(line,"%!PS-Adobe-")) {
+    if (iscomment(line,"%!PS-Adobe-") || iscomment(line,"%!PS")) {
 	doc = (struct document *) malloc(sizeof(struct document));
 	if (doc == NULL) {
 	    fprintf(stderr, "Fatal Error: Dynamic memory exhausted.\n");
@@ -372,8 +373,29 @@ psscan(FILE **fileP, const char *filename, const char *tmpprefix,
 	goto scan_ok;
 /* end of added code section */
 /* end of patch */
-	} else {
-	return(NULL);
+    } else {
+      /* There are postscript documents that do not have
+	 %PS at the beginning, usually unstructured. We should GS decide
+	 For instance, the tech reports at this university:
+
+	 http://svrc.it.uq.edu.au/Bibliography/svrc-tr.html?94-45
+
+	 add ugly PostScript before the actual document. 
+
+	 GS and gv is
+	 able to display them correctly as unstructured PS.
+
+	 In a way, this makes sense, a program PostScript does not need
+	 the !PS at the beginning.
+
+      */
+	doc = (struct document *) malloc(sizeof(struct document));
+	if (doc == NULL) {
+	    fprintf(stderr, "Fatal Error: Dynamic memory exhausted.\n");
+	    exit(-1);
+	}
+	memset(doc, 0, sizeof(struct document));
+	return(doc);
     }
 
     preread = 0;
@@ -1158,7 +1180,9 @@ psfree(doc)
     int i;
 
     if (doc) {
+      /*
     printf("This document exists\n");
+      */
 	for (i=0; i<doc->numpages; i++) {
 	    if (doc->pages[i].label) free(doc->pages[i].label);
 	}
